﻿using Microsoft.AspNetCore.Mvc.ModelBinding;
using Microsoft.Extensions.Logging;
using System.Threading.Tasks;

namespace Falcon.SugarApi.ClaimTicket
{
    /// <summary>
    /// 模型绑定
    /// </summary>
    public class ClaimTicketModelBinding:IModelBinder
    {
        /// <summary>
        /// 配置
        /// </summary>
        public ClaimTicketOptions Options { get; set; }
        /// <summary>
        /// 票据创建器
        /// </summary>
        public ITicketBuilder TicketBuilder { get; }
        /// <summary>
        /// 日志记录
        /// </summary>
        public ILogger Logger { get; set; }

        /// <summary>
        /// 通过票据创建器创建模型绑定
        /// </summary>
        /// <param name="options">配置</param>
        /// <param name="ticketBuilder">票据创建器</param>
        /// <param name="logger"></param>
        public ClaimTicketModelBinding(
            ClaimTicketOptions options,
            ITicketBuilder ticketBuilder,ILogger<ClaimTicketModelBinding> logger) {
            this.Options = options;
            TicketBuilder = ticketBuilder;
            this.Logger = logger;
        }

        Task IModelBinder.BindModelAsync(ModelBindingContext bindingContext) {
            if(!bindingContext.ModelType.IsAssignableTo(typeof(UserTicket))) {
                return FailBind(bindingContext);
            }
            var token = bindingContext.ValueProvider.GetValue(this.Options.HttpHeaderKey).ToString();
            if(token.IsNullOrEmpty()) {
                token = bindingContext.HttpContext.Request.Headers[this.Options.HttpHeaderKey].ToString();
            }
            UserTicket? model;
            try {
                model = this.TicketBuilder.GetUser(token);
            }
            catch(System.Exception ex) {
                this.Logger.LogError($"通过token生成用户时发生错误\n{ex}");
                return FailBind(bindingContext);
            }
            if(model == null) {
                return FailBind(bindingContext);
            }

            bindingContext.Model = model;
            bindingContext.Result = ModelBindingResult.Success(bindingContext.Model);

            return Task.CompletedTask;
        }

        private static Task FailBind(ModelBindingContext bindingContext) {
            bindingContext.Result = ModelBindingResult.Failed();
            return Task.CompletedTask;
        }
    }
}
